# moved to app-juices:p2t/scripts/predict_latex.py
# coding: utf-8
import time
from texify.inference import batch_inference
from texify.model.model import load_model
from texify.model.processor import load_processor
from PIL import Image

model = load_model()
processor = load_processor()
# img = Image.open("docs/examples/mixed.jpg")  # Your image name here
img_fps = [
    # 'data/call_images_formula/2023-08-14_2023-08-20/call-7PWyE5SRT5WHoxT30HgnklLf-YulVGmI_3.jpg',
    # 'data/call_images/2023-08-14_2023-08-20/call-FOgHhf7nqSyPvfVHStn1EVzlKrm0jjCx.jpg',
    # 'data/call_images_formula/2023-08-14_2023-08-20/call-__A94VFOt0gK8WwZ1eYtjVPyqq1ZgYAD_0.jpg',
    # 'data/call_images_formula/2023-08-14_2023-08-20/call-b5tzqng30ICprcGgLAxUkPpCaBI4MYx2_2.jpg',
    # 'data/call_images/2023-08-14_2023-08-20/call-0zcJMhzMndrTDb2HFIHPUCi470qrihhQ.jpg',
    # 'data/call_images_formula/2023-08-14_2023-08-20/call-dRtAWTwTDNQJQdhntWSYWmU3QaF4VFnK_0.jpg',
    # 'data/call_images_formula/2023-08-14_2023-08-20/call-hS-gUgWvRPZ76fma5RgWcfY4LBBTWDVz_0.jpg',
    '/Users/king/Documents/WhatIHaveDone/Test/pix2text/data/call_images_formula/2023-08-14_2023-08-20/call-t-Rue5yGQ4p_UrEoqFc2Gj-HW9kVsyrZ_23.jpg',
    '/Users/king/Documents/WhatIHaveDone/Test/apps-juice/p2t/out.jpg',
]
# img = Image.open("data/call_images/2023-08-14_2023-08-20/call-AczkIZ-vIHzpyt_lWllmUKKo3KDHNocy.jpg")  # Your image name here
img_list = [Image.open(fp) for fp in img_fps]
start_time = time.time()
results = batch_inference(img_list, model, processor)
end_time = time.time()
print("time:", end_time - start_time)
print('\n\n'.join(results))



# coding: utf-8
# 生成检测结果（json格式）文件，这个文件可以导入到label studio中，生成待标注的任务
from collections import OrderedDict
from glob import glob
import json
from argparse import ArgumentParser
import hashlib
import tqdm

from PIL import Image
import cv2
from cnstd import LayoutAnalyzer
from pix2text.utils import read_img


def to_json(total_width, total_height, box_type, x0, y0, w, h, _id):
    return {
        "original_width": total_width,
        "original_height": total_height,
        "image_rotation": 0,
        "value": {
            "x": x0,
            "y": y0,
            "width": w,
            "height": h,
            "rotation": 0,
            "rectanglelabels": [box_type],
        },
        "id": str(_id),
        "from_name": "label",
        "to_name": "image",
        "type": "rectanglelabels",
        "origin": "manual",
    }


def deduplicate_images(img_dir):
    def calculate_image_hash(image_path):
        # with open(img_fp, 'rb') as f:
        #     image_data = f.read()
        #     return hashlib.md5(image_data).hexdigest()
        image = cv2.imread(image_path)
        gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
        resized = cv2.resize(gray, (8, 8), interpolation=cv2.INTER_AREA)
        _, threshold = cv2.threshold(resized, 0, 255, cv2.THRESH_BINARY + cv2.THRESH_OTSU)

        return sum([2 ** i for (i, v) in enumerate(threshold.flatten()) if v])

    img_fp_list = glob('{}/*g'.format(img_dir), recursive=True)
    print(f'{len(img_fp_list)} images found in {img_dir}')
    outs = OrderedDict()
    for img_fp in tqdm.tqdm(img_fp_list):
        img_hash = calculate_image_hash(img_fp)

        # 将特征值与文件名存储在字典中
        if img_hash not in outs:
            outs[img_hash] = img_fp
    print(f'{len(outs)} different images kept after deduplication')
    return list(outs.values())


def main():
    parser = ArgumentParser()
    parser.add_argument(
        '-i', '--img-dir', type=str, required=True, help='image directory'
    )

    parser.add_argument(
        '-t',
        '--model-type',
        type=str,
        default='yolov7',
        help='模型类型。当前支持 [`yolov7_tiny`, `yolov7`]',
    )
    parser.add_argument(
        '-p',
        '--model-fp',
        type=str,
        default='epoch_124-mfd.pt',
        help='使用训练好的模型。默认为 `None`，表示使用系统自带的预训练模型',
    )

    parser.add_argument(
        '-o',
        '--out-json-fp',
        type=str,
        default='prediction_results.json',
        help='output json file',
    )
    args = parser.parse_args()
    img_dir = args.img_dir

    analyzer = LayoutAnalyzer(model_name='mfd', model_type='yolov7', model_fp=args.model_fp)

    # 遍历指定目录下的所有图片文件
    # img_fp_list = glob('{}/*g'.format(img_dir), recursive=True)
    # print(f'{len(img_fp_list)} images found in {img_dir}')
    img_fp_list = deduplicate_images(img_dir)

    total_json = []
    num_boxes = 0
    for img_fp in tqdm.tqdm(img_fp_list):
        img0 = read_img(img_fp, return_type='Image')
        width, height = img0.size
        out = analyzer.analyze(img0, resized_shape=608)

        results = []
        for box_info in out:
            num_boxes += 1
            # box with 4 points to (x0, y0, w, h)
            box = box_info['box']
            w = box[2][0] - box[0][0]
            h = box[2][1] - box[0][1]
            info = to_json(
                width,
                height,
                box_info['type'],
                100 * box[0][0] / width,
                100 * box[0][1] / height,
                100 * w / width,
                100 * h / height,
                num_boxes,
            )
            results.append(info)

        predictions = [{"model_version": "one", "score": 0.5, "result": results}]
        data = {
            # "image": img_fp,
            "image": "/data/local-files/?d=" + img_fp,
        }
        total_json.append({"data": data, "predictions": predictions})

    json.dump(total_json, open(args.out_json_fp, 'w'), indent=2, ensure_ascii=False)


if __name__ == '__main__':
    main()



# coding: utf-8
# [Pix2Text](https://github.com/breezedeus/pix2text): an Open-Source Alternative to Mathpix.
# Copyright (C) 2022-2024, [Breezedeus](https://www.breezedeus.com).

COLOR_MAPPING = {
    'general': '#009933',
    'english': '#3399ff',
    'formula': '#ff8000',
    'hybrid': '#009999',
    'text': '#3399ff',
    'isolated': '#ff8000',
    'text-embed': '#009999',
}

def render_html(newest_fp, image_type, text, out_html_fp):
    html_str = """
<!DOCTYPE html>
<html>
  <head>
    <link
      rel="stylesheet"
      href="https://cindyjs.org/dist/v0.8/katex/katex.min.css"
    />
    <script
      type="text/javascript"
      src="https://cindyjs.org/dist/v0.8/katex/katex.min.js"
    ></script>
    <script
      type="text/javascript"
      src="https://cindyjs.org/dist/v0.8/webfont.js"
    ></script>
    <style>
      body {
        /* background-color: rgb(154, 183, 249); */
      }
      #latex {
        min-height: auto;
        margin-left: auto;
        margin-right: auto;
        width: 760px;
      }

      #textarea {
        margin-left: auto;
        margin-right: auto;
        display: block;
      padding: 16px;
        /* height: 200px; */
      }

      body {
        display: flex;
        flex-direction: column;
        align-items: center;
      }

      .img-container {
        max-width: 760px;
      }
      .img-container img {
        max-width: 100%
      }
            .row {
        display: flex;
        gap: 1em;
        width: 760px;
      }
      .row textarea {
        flex: 1;
      }
      .row .col {
        width: 80px;
        display: flex;
        flex-direction: column;
        gap: 0.5em;
      }

      .container {
        width: 760px;
        position: relative;
        display: flex;
        flex-direction: column;
        align-items: center;
      }
      .container .refresh {
        position: absolute;
        right: 0;
        top: 0;
        padding: 4px 8px;
      }
      .btn {
        font-size: 16px;
        padding: 4px 2px;
        border: none;
        border-radius: 4px;
        font-weight: bolder;
        cursor: pointer;
        color: rgb(62, 59, 59);
        background-color: #b3b3cc
      }
      .img-type {
        font-size: 16px;
        padding: 4px 6px;
        border: none;
        border-radius: 4px;
        font-weight: bolder;
        color: #ffffff
      }
      .cp {
        background-color: #f07878;
      }
      .cpd {
        background-color:   #a7d797;
      }
      .cpdd {
        background-color: #74a7f9;
      }
    </style>
  </head>

  <body class="with-footer">
    <div class="container">
    <h1 align="center"><a href="https://github.com/breezedeus/pix2text">Pix2Text</a>: a free tool like Mathpix</h1>
    <h2 align="center">Screenshot</h2>
    <div class="img-container">
    """
    html_str += fr'<img src="{newest_fp}" />' + '\n </div>'
    html_str += """
    <button class="refresh btn" onClick="document.location.reload()">Refresh</button>
    </div>
    <hr />

    <h2 align="center">Results</h2>
    """

    html_str += r'<strong>Image Type: </strong>' \
                fr'<div class="img-type" style="background:{COLOR_MAPPING[image_type]}"> ' \
                fr'{image_type} </div>' + '\n'

    if image_type in ('formula', 'hybrid'):
        html_str += '<div id="latex"></div>'

    html_str += """

    <hr>

    <div class="row">

    """

    html_str += '\n<textarea id="textarea" rows="10">' + fr"{text}" + '</textarea>'

    html_str += """
          <div class="col">
        <button class="btn" type="button" onclick="copyTex()">Copy</button>
        <button class="btn" type="button" onclick="copyTexD()">$Copy$</button>
        <button class="btn" type="button" onclick="copyTexDD()">$$Copy$$</button>
      </div>
    </div>


    <script type="text/javascript">
      const textarea = document.querySelector("#textarea");
      const render = () => {
        var elt = document.createElement("div");
        elt.id = "latex";
        try {
          katex.render(textarea.value, elt, { displayMode: "display" });
        } catch (err) {
          console.error(err);
        }
        document.body.replaceChild(elt, document.querySelector("#latex"));
      };

      textarea.onblur = render;
      render();
    </script>

    <script type="text/javascript">
      function copy(text) {
        navigator.permissions
          .query({ name: "clipboard-write" })
          .then((result) => {
            if (result.state == "granted" || result.state == "prompt") {
              navigator.clipboard.writeText(text).then(() => {
                /* do nothing */
              });
            }
          });
      }
      function copyTex() {
        const texText = document.querySelector("#textarea").textContent;
        copy(texText);
      }
      function copyTexD() {
        const texText = document.querySelector("#textarea").textContent;
        copy(`$${texText}$`);
      }
      function copyTexDD() {
        const texText = document.querySelector("#textarea").textContent;
        copy(`$$${texText}$$`);
      }
    </script>

    <script>
      (async function callApi() {
        const resp = await fetch('/api/ocr')
        const data = await resp.json()
        console.log(data)
      })()
    </script>
  </body>
</html>
    """

    with open(out_html_fp, 'w', encoding='utf-8') as f:
        f.writelines(html_str)



# coding: utf-8
# [Pix2Text](https://github.com/breezedeus/pix2text): an Open-Source Alternative to Mathpix.
# Copyright (C) 2022-2024, [Breezedeus](https://www.breezedeus.com).

from .utils import read_img, set_logger, merge_line_texts
from .render import render_html
from .doc_xl_layout import DocXLayoutParser
from .latex_ocr import LatexOCR
from .text_formula_ocr import TextFormulaOCR
from .table_ocr import TableOCR
from .pix_to_text import Pix2Text



# coding: utf-8

CATEGORY_MAPPINGS = {
    "person": "人",
    "bicycle": "自行车",
    "car": "汽车",
    "motorcycle": "摩托车",
    "airplane": "飞机",
    "bus": "公车",
    "train": "火车",
    "truck": "卡车",
    "boat": "船",
    "traffic light": "交通灯",
    "fire hydrant": "消火栓",
    "stop sign": "车站牌",
    "parking meter": "停车计时器",
    "bench": "长椅",
    "bird": "鸟",
    "cat": "猫",
    "dog": "狗",
    "horse": "马",
    "sheep": "羊",
    "cow": "牛",
    "elephant": "大象",
    "bear": "熊",
    "zebra": "斑马",
    "giraffe": "长颈鹿",
    "backpack": "背包",
    "umbrella": "雨伞",
    "handbag": "手提包",
    "tie": "领带",
    "suitcase": "手提箱",
    "frisbee": "飞盘",
    "skis": "滑雪板",
    "snowboard": "滑雪板",
    "sports ball": "运动球",
    "kite": "风筝",
    "baseball bat": "棒球棒",
    "baseball glove": "棒球手套",
    "skateboard": "滑板",
    "surfboard": "冲浪板",
    "tennis racket": "网球拍",
    "bottle": "瓶子",
    "wine glass": "酒杯",
    "cup": "杯子",
    "fork": "叉子",
    "knife": "刀",
    "spoon": "勺子",
    "bowl": "碗",
    "banana": "香蕉",
    "apple": "苹果",
    "sandwich": "三明治",
    "orange": "橙子",
    "broccoli": "西兰花",
    "carrot": "胡萝卜",
    "hot dog": "热狗",
    "pizza": "披萨",
    "donut": "甜甜圈",
    "cake": "蛋糕",
    "chair": "椅子",
    "couch": "沙发",
    "potted plant": "盆栽",
    "bed": "床",
    "dining table": "餐桌",
    "toilet": "厕所",
    "tv": "电视机",
    "laptop": "笔记本",
    "mouse": "鼠标",
    "remote": "遥控器",
    "keyboard": "键盘",
    "cell phone": "手机",
    "microwave": "微波炉",
    "oven": "烤箱",
    "toaster": "烤面包机",
    "sink": "水槽",
    "refrigerator": "冰箱",
    "book": "书",
    "clock": "时钟",
    "vase": "花瓶",
    "scissors": "剪刀",
    "teddy bear": "泰迪熊",
    "hair drier": "电吹风",
    "toothbrush": "牙刷",
    "N/A": "-",
}
